/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2009 EADS DEFENCE AND SECURITY SYSTEMS
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 */

package org.ow2.weblab.core.helper;

import java.util.Locale;
import java.util.Map;

import org.apache.commons.logging.LogFactory;
import org.ow2.weblab.core.extended.exception.WebLabUncheckedException;
import org.ow2.weblab.core.extended.properties.PropertiesLoader;

/**
 * Abstract class which enable to load dynamically the implementation. It aims to load JAVA bean in accordance to a configuration file.
 * 
 * @author EADS WebLab Team
 * @date 2008-03-18
 */
public abstract class BeanHelper {

	/**
	 * Current loaded instance.
	 */
	private static BeanHelper instance;

	/**
	 * The properties file used.
	 */
	public static final String FILENAME = "bean-helper.properties";

	/**
	 * Implementation class name.
	 */
	private static String implClassName;

	/**
	 * The default constructor to be called by implementations.
	 */
	protected BeanHelper() {
		// Only used by sub-classes.
	}

	/**
	 * @return The current <code>BeanHelper</code> instance.
	 */
	public static BeanHelper getInstance() {
		if (BeanHelper.instance == null) {
			BeanHelper.instance = BeanHelper.loadImpl();
		}
		return BeanHelper.instance;
	}

	/**
	 * @return The loaded implementation.
	 */
	private static BeanHelper loadImpl() {
		BeanHelper factory;

		final String className = BeanHelper.class.getSimpleName().toLowerCase(Locale.ENGLISH);


		final Map<String, String> map = PropertiesLoader.loadProperties(BeanHelper.FILENAME, BeanHelper.class);
		BeanHelper.implClassName = map.get(className);
		if (BeanHelper.implClassName == null) {
			throw new WebLabUncheckedException("Cannot instantiate " + BeanHelper.class.getName()
					+ ". It was not defined in " + BeanHelper.FILENAME);
		}
		try {
			final Class<?> impl = BeanHelper.class.getClassLoader().loadClass(BeanHelper.implClassName);
			LogFactory.getLog(BeanHelper.class).debug("Load: '" + BeanHelper.implClassName + "'.");
			final Object o = impl.newInstance();
			if (BeanHelper.class.isAssignableFrom(impl)) {
				factory = BeanHelper.class.cast(o);
			} else {
				throw new ClassCastException("Unable to cast the loaded " + "bean into a " + className);
			}
			return factory;
		} catch (final ClassNotFoundException cnfe) {
			throw new WebLabUncheckedException("Cannot instantiate " + BeanHelper.class.getName() + ".", cnfe);
		} catch (final InstantiationException ie) {
			throw new WebLabUncheckedException("Cannot instantiate " + BeanHelper.class.getName() + ".", ie);
		} catch (final IllegalAccessException iae) {
			throw new WebLabUncheckedException("Cannot instantiate " + BeanHelper.class.getName() + ".", iae);
		}
	}

	/**
	 * Return the current instance of Bean factory with respect to the configuration file.
	 * 
	 * @param path
	 *            The path to the configuration file.
	 * @return The instance of <code>BeanHelper</code> in accordance with the configuration file.
	 * @see #getSpecificInstance(String, boolean)
	 */
	public abstract BeanHelper getSpecificInstance(final String path);

	/**
	 * This method is similar to {@link #getSpecificInstance(String)} but when <code>name</code> is <code>true</code>, it's loaded from the classpath, without
	 * the whole path if possible.<br />
	 * Return the current instance of Bean factory with respect to the configuration file.
	 * 
	 * @param fileNameOrPath
	 *            The name of (or the path to) the configuration file.
	 * @param isName
	 *            Whether <code>fileNameOrPath</code> is a name or a path.
	 * @return The instance of <code>BeanHelper</code> in accordance with the configuration file.
	 * @see #getSpecificInstance(String)
	 */
	public abstract BeanHelper getSpecificInstance(final String fileNameOrPath, final boolean isName);

	/**
	 * Get the bean defined by its name in the configuration file.
	 * 
	 * @param <T>
	 *            The class of the bean to be instantiated
	 * @param name
	 *            The name of the bean to retrieve.
	 * @param beanClass
	 *            The class of the bean to be instantiated.
	 * @return The bean defined with <code>name</code> in the <code>BeanHelper</code>.
	 * @throws WebLabUncheckedException
	 *             If bean does not exist in configuration file loaded.
	 */
	public abstract <T> T getBean(final String name, final Class<T> beanClass);

	/**
	 * @param name
	 *            The name of the bean to test existence in the configuration file.
	 * @param beanClass
	 *            The class of the bean.
	 * @return Whether or not a bean named <code>name</code> and of a type (or subtype, or implementation) or <code>beanClass</code> exists in configuration
	 *         file.
	 */
	public abstract boolean isBeanDefined(final String name, final Class<?> beanClass);

}